package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetOuListTreeOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.*;
import pps.core.base.mapper.BaseLineAttenuationRateMapper;
import pps.core.base.mapper.BaseLineAttenuationRateViewMapper;
import pps.core.base.mapper.BaseLineMapper;
import pps.core.base.mapper.BaseLineViewMapper;
import pps.core.base.service.data.base_line.*;
import pps.core.base.service.data.base_line_attenuation_rate.CreateBaseLineAttenuationRateInput;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysAreaEnt;
import pps.core.system.mapper.SysAreaMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@XService
public class BaseLineService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseLine(XContext context, CreateBaseLineInput input) {
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        BaseLineEnt entity = new BaseLineEnt();
        XCopyUtils.copyObject(input, entity);
        if (entity.getParentId() == null)
            entity.setParentId(0);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        mapper.insert(entity);

        if (!CollectionUtils.isEmpty(input.getList())) {
            BaseLineAttenuationRateViewMapper rateMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            List<BaseLineAttenuationRateView> list = new ArrayList<>();
            for (CreateBaseLineAttenuationRateInput rateInput : input.getList()) {
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput, BaseLineAttenuationRateView.class);
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                ent.setCreateTime(now);
                ent.setType(0);
                ent.setLineId(entity.getId());
                list.add(ent);
            }
            rateMapper.batchInsert(list);
        }
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseLine(XContext context, UpdateBaseLineInput input) {
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        if (entity.getParentId() == null)
            entity.setParentId(0);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
        mapper.updateById(entity);
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        QueryWrapper<BaseLineAttenuationRateEnt> queryRateWrapper = new QueryWrapper<>();
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getLineId, entity.getId());
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getType, 0);
        rateMapper.delete(queryRateWrapper);
        if (!CollectionUtils.isEmpty(input.getList())) {
            BaseLineAttenuationRateViewMapper rateViewMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            List<BaseLineAttenuationRateView> list = new ArrayList<>();
            for (CreateBaseLineAttenuationRateInput rateInput : input.getList()) {
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput, BaseLineAttenuationRateView.class);
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                ent.setCreateTime(now);
                ent.setLineId(entity.getId());
                ent.setType(0);
                list.add(ent);
            }
            rateViewMapper.batchInsert(list);
        }
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseLine(XContext context, DeleteBaseLineInput input) {
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XSingleResult<GetBaseLineOutput> getBaseLine(XContext context, GetBaseLineInput input) {
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView lineView = new BaseLineView();
        lineView.setId(String.valueOf(input.getId()));
        BaseLineView entity = mapper.selectOne(lineView);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseLineOutput output = new GetBaseLineOutput();
        XCopyUtils.copyObject(entity, output);
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        QueryWrapper<BaseLineAttenuationRateEnt> queryRateWrapper = new QueryWrapper<>();
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getLineId, entity.getId());
        queryRateWrapper.lambda().eq(BaseLineAttenuationRateEnt::getType, 0);
        List<BaseLineAttenuationRateEnt> list = rateMapper.selectList(queryRateWrapper);
        if (!CollectionUtils.isEmpty(list)) {
            List<CreateBaseLineAttenuationRateInput> rateInputs = XCopyUtils.copyNewList(list, CreateBaseLineAttenuationRateInput.class);
            output.setList(rateInputs);
        }
        if (output.getParentId() == 0)
            output.setParentId(null);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> queryBaseLineList(XContext context, QueryBaseLineListInput input) {
        PageHelper.startPage(input.getPage(), input.getLimit());
        //获取组织机构下的子集
        SysOrganizationCloudService cloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        if (XStringUtils.isNotEmpty(input.getOuId())) {
            ouIdInput.setOuId(input.getOuId());
        } else {
            ouIdInput.setOuId("00000000-0000-0000-0000-000000000000");
        }
        XListResult<GetOuListTreeOutput> outputXListResult = cloudService.getAllOuListByOuId(context, ouIdInput);
        outputXListResult.throwIfFail();
        List<GetOuListTreeOutput> ouList = outputXListResult.getResult();
        List<String> ouIdList = new ArrayList<>();
        if (CollUtil.isNotEmpty(ouList)) {
            ouIdList = ouList.stream().map(GetOuListTreeOutput::getOuId).collect(Collectors.toList());
        }
        BaseLineViewMapper viewMapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input, BaseLineView.class);
        view.setOuIdList(ouIdList);
        List<BaseLineView> list = viewMapper.selectListNew(view);
        PageInfo<BaseLineView> pageInfo = new PageInfo(list);
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineOutput.class);
        //获取线路、电网字典
        List<QuerySysDictionaryViewOutput> gridTypeList = getDictList(context, "grid_type");
        List<QuerySysDictionaryViewOutput> lineTypeList = getDictList(context, "line_type");
        for (QueryBaseLineOutput item : outputs) {
            item.setGridTypeName(getDictName(item.getGridType(), gridTypeList));
            item.setLineTypeName(getDictName(item.getLineType(), lineTypeList));
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> queryBaseLine(XContext context, QueryBaseLineInput input) {
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input, BaseLineView.class);
        List<BaseLineView> list = mapper.selectList(view);
        PageInfo<BaseLineView> pageInfo = new PageInfo(list);
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineAllOutput> queryAllBaseLine(XContext context, QueryBaseLineInput input) {
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input, BaseLineView.class);
        List<BaseLineAllView> list = mapper.selectDataList(view);
        PageInfo<BaseLineAllView> pageInfo = new PageInfo(list);
        List<QueryBaseLineAllOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineAllOutput.class);
        SystemAreaService areaService = context.getBean(SystemAreaService.class);
//        XListResult<GetSysAreaOutput> areaOutputXListResult = areaService.getSysAreaList(context ,new GetSysAreaInput());
//        areaOutputXListResult.throwIfFail();
//        List<GetSysAreaOutput> areaOutputs = areaOutputXListResult.getResult();
        SysAreaMapper areaMapper = context.getBean(SysAreaMapper.class);
        List<SysAreaEnt> areaEntList = areaMapper.selectList(new QueryWrapper<>());
        outputs.stream().forEach(item -> {
            if (XStringUtils.isNotEmpty(item.getProvince())) {
                System.out.println(item.getProvince());
                SysAreaEnt areaEnt = areaEntList.stream().filter(area -> item.getProvince().equals(area.getId() + "")).findFirst().orElse(null);
                if (areaEnt != null)
                    item.setProvince(getParentNameStr(areaEntList, areaEnt));
            }
        });
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> dynamicQueryBaseLine(XContext context, CustomQueryInput input) {
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        return XMapperHelper.query(mapper, input, BaseLineEnt.class, QueryBaseLineOutput.class);
    }

    public List<QuerySysDictionaryViewOutput> getDictList(XContext context, String alias) {
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey, List<QuerySysDictionaryViewOutput> dictList) {
        if (StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)) {
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if (dict.getDicKey().equals(dictKey)) {
                return dict.getDicName();
            }
        }
        return "";
    }

    @XApiAnonymous
    @XApiPost
    public XListResult<GetBaseLineTreeOutput> getBaseLineTree(XContext context, GetBaseLineInput input) {
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        //queryWrapper.lambda().in(BaseLineEnt::getWellOuId , ouIdList);
        List<BaseLineEnt> list = mapper.selectList(queryWrapper);
        List<GetBaseLineTreeOutput> outputs = new ArrayList<>();
        for (BaseLineEnt ent : list) {
            GetBaseLineTreeOutput output = new GetBaseLineTreeOutput();
            output.setParentId(ent.getParentId());
            output.setValue(ent.getId());
            output.setLabel(ent.getLineName());
            outputs.add(output);
        }
        if (!CollectionUtils.isEmpty(outputs))
            selectBaseLineTree(outputs);
        return XListResult.success(outputs);
    }


    public List<GetBaseLineTreeOutput> selectBaseLineTree(List<GetBaseLineTreeOutput> list) {
        //操作所有组织机构数据
        Map<Integer, List<GetBaseLineTreeOutput>> groupMap = list.stream().collect(Collectors.groupingBy(GetBaseLineTreeOutput::getParentId));
        list.forEach(organization -> {
            organization.setChildren(groupMap.get(organization.getValue()));
        });
        List<GetBaseLineTreeOutput> collect = list.stream().filter(organization -> organization.getParentId() == 0).collect(Collectors.toList());
        return collect;
    }

    @XApiAnonymous
    @XApiPost
    public XListResult<GetBaseLineOutput> getBaseLineList(XContext context, GetBaseLineInput input) {
        //获取组织机构下的子集
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        // queryWrapper.lambda().in(BaseLineEnt::getWellOuId , ouIdList);
        List<BaseLineEnt> list = mapper.selectList(queryWrapper);
        List<GetBaseLineOutput> outputs = XCopyUtils.copyNewList(list, GetBaseLineOutput.class);
        return XListResult.success(outputs);
    }

    /**
     * 根据子节点获取最上层节点
     *
     * @return
     */
    public static String getParentNameStr(List<SysAreaEnt> areaList, SysAreaEnt child) {
        String parentNameStr = child.getName();
        Integer parentId = child.getParentId();
        if (parentId == 120) {

        } else {
            SysAreaEnt parent = areaList.stream().filter(item -> item.getId().equals(parentId)).findFirst().orElse(null);
            String nameStr = getParentNameStr(areaList, parent);
            parentNameStr = nameStr + "/" + parentNameStr;
        }
        return parentNameStr;
    }
}
