package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceCalibrationCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 定期校准定时任务
 *
 * @author ZWT
 * @date 2023/09/20 09:20
 */
@XText("长期间开优化定时任务")
@XService
public class SpaceCalibrationJob implements XJob {

    /**
     * 每天00:15执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceCalibrationJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceCalibrationJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                ISpaceCalibrationCloudService service = xContext.getBean(ISpaceCalibrationCloudService.class);
                XServiceResult result = service.calibrateJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ SpaceCalibrationJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceCalibrationJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
