package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceOptimizeMidCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 中短期间开优化定时任务
 *
 * @author ZWT
 * @date 2023/09/18 13:58
 */
@XText("中短期间开优化定时任务")
@XService
public class SpaceOptimizeMidJob implements XJob {

    /**
     * 每周一执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_MONDAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceOptimizeMidJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceOptimizeMidJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                ISpaceOptimizeMidCloudService service = xContext.getBean(ISpaceOptimizeMidCloudService.class);
                XServiceResult result = service.optimizeMidJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ SpaceOptimizeMidJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceOptimizeMidJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}