package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 短期间开优化定时任务
 *
 * @author ZWT
 * @date 2023/09/20 11:39
 */
@XText("短期间开优化定时任务")
@XService
public class SpaceOptimizeShortJob implements XJob {

    /**
     * 每一天 23:50:00 执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.END_OF_DAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceOptimizeShortJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceOptimizeShortJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                ISpaceOptimizeShortCloudService service = xContext.getBean(ISpaceOptimizeShortCloudService.class);
                XServiceResult result = service.optimizeShortJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ SpaceOptimizeShortJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceOptimizeShortJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}