package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 天气数据邮件接收定时任务
 *
 * @author ZWT
 * @date 2023/11/13 17:15
 */
@XText("天气数据邮件接收定时任务")
@XService
public class WeatherReceiveJob implements XJob {

    /**
     * 半小时执行一次
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.HALF_HOUR)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WeatherReceiveJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "WeatherReceiveJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                IBaseWeatherCloudService service = xContext.getBean(IBaseWeatherCloudService.class);
                XServiceResult result = service.weatherDataProcess(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WeatherReceiveJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WeatherReceiveJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
