package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 日耗电日产液信息
 *
 * @author ZWT
 * @date 2024/03/18 14:52
 */
@XText("日耗电日产液信息定时任务")
@XService
public class WellTechDailyJob implements XJob {

    /**
     * 每天0点执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_ZERO)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WellTechDailyJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "WellTechDailyJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                IPlantPredictedPowerCloudService service = xContext.getBean(IPlantPredictedPowerCloudService.class);
                XServiceResult result = service.wellTechDailyJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WellTechDailyJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WellTechDailyJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
