package pps.core.prediction.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.prediction.entity.PlantPredictedPowerMidTermEnt;
import pps.core.prediction.mapper.PlantPredictedPowerMidTermMapper;
import pps.core.prediction.service.data.plant_predicted_power_mid_term.GetPlantPredictedPowerMidTermInput;
import pps.core.prediction.service.data.plant_predicted_power_mid_term.GetPlantPredictedPowerMidTermOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.Date;
import java.util.List;


@XService
@XText("电站预测功率中期240小时数据")
public class PlantPredictedPowerMidTermService {

    @XApiGet
    @XText("根据电站预测15天的预测数据")
    public XListResult<GetPlantPredictedPowerMidTermOutput> getFifteenPlantPredictedPowerMidTermList(XContext context, GetPlantPredictedPowerMidTermInput input){
        PlantPredictedPowerMidTermMapper mapper = context.getBean(PlantPredictedPowerMidTermMapper.class);
        QueryWrapper<PlantPredictedPowerMidTermEnt> queryWrapper = setQueryWrapper(input.getPlantId() , input.getStartTime() , 15);
        List<PlantPredictedPowerMidTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerMidTermOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerMidTermOutput.class);
        return XListResult.success(outputs);
    }


    @XApiGet
    @XText("根据电站预测10天的预测数据")
    public XListResult<GetPlantPredictedPowerMidTermOutput> getTenPlantPredictedPowerMidTermList(XContext context, GetPlantPredictedPowerMidTermInput input){
        PlantPredictedPowerMidTermMapper mapper = context.getBean(PlantPredictedPowerMidTermMapper.class);
        QueryWrapper<PlantPredictedPowerMidTermEnt> queryWrapper = setQueryWrapper(input.getPlantId() , input.getStartTime() , 10);
        List<PlantPredictedPowerMidTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerMidTermOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerMidTermOutput.class);
        return XListResult.success(outputs);
    }

    private QueryWrapper<PlantPredictedPowerMidTermEnt> setQueryWrapper(String plantId , String startTime , Integer days){
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerMidTermEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerMidTermEnt::getPlantId ,plantId);
        if(XStringUtils.isEmpty(startTime)){
            queryWrapper.lambda().ge(PlantPredictedPowerMidTermEnt::getDataDate , date);
            queryWrapper.lambda().le(PlantPredictedPowerMidTermEnt::getDataDate , XDateUtils.addDays(date , days));
        } else{
            queryWrapper.lambda().ge(PlantPredictedPowerMidTermEnt::getDataDate , startTime);
            queryWrapper.lambda().le(PlantPredictedPowerMidTermEnt::getDataDate , XDateUtils.addDays(XDateUtils.parse(startTime) , days));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerMidTermEnt::getDataDate);
        return queryWrapper;
    }
}
