package pps.core.prediction.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.service.data.plant_predicted_power_short_term.GetPlantPredictedPowerShortTermInput;
import pps.core.prediction.service.data.plant_predicted_power_short_term.GetPlantPredictedPowerShortTermOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.Date;
import java.util.List;


@XService
@XText("电站预测功率短期72小时数据")
public class PlantPredictedPowerShortTermService {

    @XApiGet
    @XText("根据电站预测3天的预测数据")
    public XListResult<GetPlantPredictedPowerShortTermOutput> getThreePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input){
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = setQueryWrapper(input.getPlantId() , input.getStartTime() , 3);
        List<PlantPredictedPowerShortTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerShortTermOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerShortTermOutput.class);
        return XListResult.success(outputs);
    }


    @XApiGet
    @XText("根据电站预测1天的预测数据")
    public XListResult<GetPlantPredictedPowerShortTermOutput> getOnePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input){
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = setQueryWrapper(input.getPlantId() , input.getStartTime() , 1);
        List<PlantPredictedPowerShortTermEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerShortTermOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerShortTermOutput.class);
        return XListResult.success(outputs);
    }

    private QueryWrapper<PlantPredictedPowerShortTermEnt> setQueryWrapper(String plantId , String startTime , Integer days){
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerShortTermEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerShortTermEnt::getPlantId ,plantId);
        if(XStringUtils.isEmpty(startTime)){
            queryWrapper.lambda().ge(PlantPredictedPowerShortTermEnt::getDataDate , date);
            queryWrapper.lambda().le(PlantPredictedPowerShortTermEnt::getDataDate , XDateUtils.addDays(date , days));
        } else{
            queryWrapper.lambda().ge(PlantPredictedPowerShortTermEnt::getDataDate , startTime);
            queryWrapper.lambda().le(PlantPredictedPowerShortTermEnt::getDataDate , XDateUtils.addDays(XDateUtils.parse(startTime) , days));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate);
        return queryWrapper;
    }
}
