package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.prediction.entity.ThirdCurrentWellConditionEnt;
import pps.core.prediction.mapper.ThirdCurrentWellConditionMapper;
import pps.core.prediction.service.data.third_current_well_condition.QueryThirdCurrentWellConditionInput;
import pps.core.prediction.service.data.third_current_well_condition.QueryThirdCurrentWellConditionOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方当日油井状态
 *
 * @author ZWT
 * @date 2024/03/19
 */
@XText("第三方当日油井状态")
@XService
public class ThirdCurrentWellConditionService {

    /**
     * 油井状态
     * POST /prediction/third-current-well-condition/query-well-real-pv-data
     * 接口ID：156558765
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156558765
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link QueryThirdCurrentWellConditionOutput}>
     */
    @XApiAnonymous
    @XText("井口生产情况--油井状态")
    @XApiPost
    public XListResult<QueryThirdCurrentWellConditionOutput> queryWellRealPvData(XContext context, QueryThirdCurrentWellConditionInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineId(input.getLineId())
                .build()
        );
        result.throwIfFail();
        List<QueryThirdCurrentWellConditionOutput> outputs;
        List<DynamicQueryBasePowerLineWellheadViewOutput> list = result.getResult();
        if (CollUtil.isEmpty(list)) {
            return XListResult.success(new ArrayList<>(0));
        }
        ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> selectList = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, list.stream()
                        .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                        .collect(Collectors.toList()))
        );
        return XListResult.success(XCopyUtils.copyNewList(selectList, QueryThirdCurrentWellConditionOutput.class));
    }
}
